@@ -1,3 +1,4 @@
+# -*- coding: future_fstrings -*-
 """This module is the heart of PyEcore. It defines all the basic concepts that
 are common to EMF-Java and PyEcore (EObject/EClass...).
 It defines the basic classes and behavior for PyEcore implementation:
@@ -124,7 +125,7 @@ class Core(object):
                          else 'default_package')
             epackage.eClass = EPackage(name=pack_name,
                                        nsPrefix=pack_name,
-                                       nsURI='http://{}/'.format(pack_name))
+                                       nsURI=f'http://{pack_name}/')
         if not hasattr(epackage, 'eURIFragment'):
             epackage.eURIFragment = eURIFragment
         cname = rcls.name if isinstance(rcls, EClassifier) else rcls.__name__
@@ -153,8 +154,8 @@ class MetaEClass(Metasubinstance):
 
     def __call__(cls, *args, **kwargs):
         if cls.eClass.abstract:
-            raise TypeError("Can't instantiate abstract EClass {0}"
-                            .format(cls.eClass.name))
+            raise TypeError("Can't instantiate "
+                            f"abstract EClass {cls.eClass.name}")
         return super().__call__(*args, **kwargs)
 
 
@@ -275,16 +276,15 @@ class EObject(ENotifer, metaclass=Metasubinstance):
             if not self.eResource or len(self.eResource.contents) == 1:
                 return '/'
             else:
-                return '/{}'.format(self.eResource.contents.index(self))
+                return f'/{self.eResource.contents.index(self)}'
         feat = self.eContainmentFeature()
         parent = self.eContainer()
         name = feat.name
         if feat.many:
             index = parent.__getattribute__(name).index(self)
-            return '{0}/@{1}.{2}' \
-                   .format(parent.eURIFragment(), name, str(index))
+            return f'{parent.eURIFragment()}/@{name}.{index}'
         else:
-            return '{0}/@{1}'.format(parent.eURIFragment(), name)
+            return f'{parent.eURIFragment()}/@{name}'
 
     def eRoot(self):
         if not self.eContainer():
@@ -310,10 +310,10 @@ class EModelElement(EObject):
             if not self.eResource or len(self.eResource.contents) == 1:
                 return '#/'
             else:
-                return '#/{}'.format(self.eResource.contents.index(self))
+                return f'#/{self.eResource.contents.index(self)}'
         parent = self.eContainer()
         if hasattr(self, 'name'):
-            return '{0}/{1}'.format(parent.eURIFragment(), self.name)
+            return f'{parent.eURIFragment()}/{self.name}'
         else:
             return super().eURIFragment()
 
@@ -414,9 +414,11 @@ class EOperation(ETypedElement):
         parameters = [x.to_code() for x in self.eParameters]
         if len(parameters) == 0 or parameters[0] != 'self':
             parameters.insert(0, 'self')
-        return """def {0}({1}):
-        raise NotImplementedError('Method {0}({1}) is not yet implemented')
-        """.format(self.normalized_name(), ', '.join(parameters))
+        norm_name = self.normalized_name()
+        parameters = ', '.join(parameters)
+        return f"""def {norm_name}({parameters}):
+        raise NotImplementedError('Method {norm_name}({parameters}) is not yet implemented')
+        """ # noqa
 
 
 class EParameter(ETypedElement):
@@ -425,9 +427,9 @@ class EParameter(ETypedElement):
 
     def to_code(self):
         if self.required:
-            return "{0}".format(self.name)
+            return f"{self.name}"
         default_value = getattr(self.eType, 'default_value', None)
-        return "{0}={1}".format(self.name, default_value)
+        return f"{self.name}={default_value}"
 
 
 class ETypeParameter(ENamedElement):
@@ -448,9 +450,8 @@ class ETypeParameter(ENamedElement):
 
     def __str__(self):
         raw_types = self.raw_types()
-        return '<{}[{}] object at {}>'.format(self.__class__.__name__,
-                                              raw_types,
-                                              hex(id(self)))
+        return (f'<{self.__class__.__name__}[{raw_types}] object '
+                f'at {hex(id(self))}>')
 
 
 class EGenericType(EObject):
@@ -538,7 +539,7 @@ class EDataType(EClassifier):
 
     def __repr__(self):
         etype = self.eType.__name__ if self.eType else None
-        return '{0}({1})'.format(self.name, etype)
+        return f'{self.name}({etype})'
 
 
 class EEnum(EDataType):
@@ -576,8 +577,8 @@ class EEnum(EDataType):
             i = literals.index(literal)
             literals.insert(0, literals.pop(i))
         else:
-            raise AttributeError('Enumeration literal {} does not exist '
-                                 'in {}'.format(value, self))
+            raise AttributeError(f'Enumeration literal {value} does not exist '
+                                 f'in {self}')
 
     def __contains__(self, key):
         if isinstance(key, EEnumLiteral):
@@ -600,7 +601,7 @@ class EEnum(EDataType):
 
     def __repr__(self):
         name = self.name or ''
-        return '{}[{}]'.format(name, str(self.eLiterals))
+        return f'{name}[{self.eLiterals}]'
 
 
 class EEnumLiteral(ENamedElement):
@@ -609,7 +610,7 @@ class EEnumLiteral(ENamedElement):
         self.value = value
 
     def __repr__(self):
-        return '{0}={1}'.format(self.name, self.value)
+        return f'{self.name}={self.value}'
 
     def __str__(self):
         return self.name
@@ -686,7 +687,7 @@ class EStructuralFeature(ETypedElement):
     def __repr__(self):
         eType = getattr(self, 'eType', None)
         name = getattr(self, 'name', None)
-        return '<{0} {1}: {2}>'.format(self.__class__.__name__, name, eType)
+        return f'<{self.__class__.__name__} {name}: {eType}>'
 
 
 class EAttribute(EStructuralFeature):
@@ -792,6 +793,9 @@ class EClass(EClassifier):
         instance.__name__ = name
         return instance
 
+    def __mro_entries__(self, old_base):
+        return (self.python_class, )
+
     def __init__(self, name=None, superclass=None, abstract=False,
                  metainstance=None, **kwargs):
         super().__init__(name, **kwargs)
@@ -800,8 +804,7 @@ class EClass(EClassifier):
 
     def __call__(self, *args, **kwargs):
         if self.abstract:
-            raise TypeError("Can't instantiate abstract EClass {0}"
-                            .format(self.name))
+            raise TypeError(f"Can't instantiate abstract EClass {self.name}")
         return self.python_class(*args, **kwargs)
 
     def allInstances(self=None, resources=None):
@@ -863,7 +866,7 @@ class EClass(EClassifier):
             return tuple(x.python_class for x in eSuperTypes)
 
     def __repr__(self):
-        return '<{0} name="{1}">'.format(self.__class__.__name__, self.name)
+        return f'<{self.__class__.__name__} name="{self.name}">'
 
     @property
     def eAttributes(self):
@@ -1224,10 +1227,13 @@ EGenericType.eUpperBound = EReference('eUpperBound', EGenericType)
 EGenericType.eLowerBound = EReference('eLowerBound', EGenericType)
 
 register_classifier = Core.register_classifier
+
+
 def register_metaclass(c, metaclass=MetaEClass, *args, **kwargs):
     register_classifier(c, *args, **kwargs)
     c.__class__ = metaclass
 
+
 eClass = EPackage(name=name, nsURI=nsURI, nsPrefix=nsPrefix)
 register_metaclass(EObject, promote=True, abstract=True,
                    metaclass=Metasubinstance)
