@@ -1,3 +1,4 @@
+# -*- coding: future_fstrings -*-
 """ The resource module proposes all the concepts that are related to Resource
 handling. A Resource represents a special model container that can be
 serialized. Many ``Resource`` can be contained in a ``ResourceSet``, and
@@ -12,6 +13,9 @@ from collections import ChainMap
 from .. import ecore as Ecore
 from ..innerutils import ignored
 from abc import abstractmethod
+from urllib.parse import urljoin
+from functools import lru_cache
+
 
 global_registry = {}
 global_uri_mapper = {}
@@ -214,7 +218,7 @@ class HttpURI(URI):
         raise NotImplementedError('Cannot create an outstream for HttpURI')
 
     def apply_relative_from_me(self, relative_path):
-        return self.plain
+        return urljoin(self.normalize(), relative_path)
 
 
 # class StdioURI(URI):
@@ -346,7 +350,7 @@ class Resource(object):
         self.listeners = []
         self._eternal_listener = []
         self._resolve_mem = {}
-        self._feature_cache = {}
+        # self._feature_cache = {}
         self.cache_enabled = False
 
     @property
@@ -425,7 +429,7 @@ class Resource(object):
             else:
                 return global_registry[nsuri]
         except KeyError:
-            raise KeyError('Unknown metamodel with uri: {0}'.format(nsuri))
+            raise KeyError(f'Unknown metamodel with uri: {nsuri}')
 
     @staticmethod
     def normalize(fragment):
@@ -459,9 +463,8 @@ class Resource(object):
                 rset.resources[original_uri] = resource
             return rset
         except Exception as e:
-            raise TypeError('Resource "{0}" cannot be resolved '
-                            'problem with "{1}"'
-                            .format(uri, e))
+            raise TypeError(f'Resource "{uri}" cannot be resolved '
+                            f'problem with "{e}"')
 
     @staticmethod
     def is_fragment_uuid(fragment):
@@ -537,7 +540,7 @@ class Resource(object):
         if obj.eResource != self:
             eclass = obj.eClass
             prefix = eclass.ePackage.nsPrefix
-            _type = '{0}:{1}'.format(prefix, eclass.name)
+            _type = f'{prefix}:{eclass.name}'
             uri_fragment = obj.eURIFragment()
             crossref = False
             if obj.eResource:
@@ -570,9 +573,9 @@ class Resource(object):
             if not uri_fragment.startswith('#'):
                 uri_fragment = '#' + uri_fragment
             if crossref:
-                return ('{0}{1}'.format(uri, uri_fragment), True)
+                return (f'{uri}{uri_fragment}', True)
             else:
-                return ('{0} {1}{2}'.format(_type, uri, uri_fragment), False)
+                return (f'{_type} {uri}{uri_fragment}', False)
         if self.use_uuid:
             self._assign_uuid(obj)
             return (obj._internal_id, False)
@@ -596,9 +599,16 @@ class Resource(object):
     def append(self, root):
         if not isinstance(root, Ecore.EObject):
             raise ValueError('The resource requires an EObject type, '
-                             'but received {0} instead.'.format(type(root)))
+                             f'but received {type(root)} instead.')
         self.contents.append(root)
         root._eresource = self
+        if root._container is not None:
+            container = root._container
+            feature = root._containment_feature
+            if feature.many:
+                container.eGet(feature).remove(root)
+            else:
+                container.eSet(feature, None)
 
     def remove(self, root):
         self.contents.remove(root)
@@ -615,11 +625,13 @@ class Resource(object):
         for x in values:
             append(x)
 
+    @lru_cache()
     def _find_feature(self, eclass, name):
-        fname = eclass.name + '#' + name
-        try:
-            return self._feature_cache[fname]
-        except KeyError:
-            feature = eclass.findEStructuralFeature(name)
-            self._feature_cache[fname] = feature
-            return feature
+        return eclass.findEStructuralFeature(name)
+        # fname = f'{eclass.name}#{name}'
+        # try:
+        #     return self._feature_cache[fname]
+        # except KeyError:
+        #     feature = eclass.findEStructuralFeature(name)
+        #     self._feature_cache[fname] = feature
+        #     return feature
